from typing import Optional

import aiohttp

from .http import HTTPClient
from .models import (LeaderboardBasedRequests, MapBasedRequests,
                     MapsetsBasedRequests, MiscBasedRequest,
                     MultiplayerBasedRequests, PlaylistBasedRequest,
                     UserBasedRequests)

__all__ = ('Quaver',)


class Quaver (
        UserBasedRequests, MapsetsBasedRequests,
        LeaderboardBasedRequests, PlaylistBasedRequest,
        MapBasedRequests, MultiplayerBasedRequests,
        MiscBasedRequest
):
    __slots__ = ('_client',)

    def __init__(self, session: Optional[aiohttp.ClientSession] = None):
        """
        The class for the Quaver API.

        Parameters
        ----------
        session: Optional[aiohttp.ClientSession]
            The session to use for the HTTPClient to send requests to the API.

        Raises
        ------
        APIDown
            If the API is down.

        Attributes
        ----------
        _client: HTTPClient
            The HTTPClient to use for sending requests to the API.
        """
        self._client = HTTPClient(session)

    async def close(self) -> None:
        """
        Closes the HTTPClient.
        """
        await self._client.close()

    async def __aenter__(self):
        return self
    
    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.close()
        



class QuaverError(Exception):
    """Base class for exceptions in this module."""
    pass

class APIDown(QuaverError):
    """Exception raised for errors in the API.

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message):
        self.message = message

class InvalidArgumentPassed(QuaverError):
    """Exception raised for errors in the API.

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message):
        self.message = message


from typing import Tuple

import pkg_resources
import argparse
import platform
import aiohttp
import quaver
import sys

def show_version() -> None:
    entries = []

    entries.append('- Python v{0.major}.{0.minor}.{0.micro}-{0.releaselevel}'.format(sys.version_info))
    version_info = quaver.version_info
    entries.append('- discord.py v{0.major}.{0.minor}.{0.micro}-{0.releaselevel}'.format(version_info))
    if version_info.releaselevel != 'final':
        pkg = pkg_resources.get_distribution('quaver.py')
        if pkg:
            entries.append(f'    - quaver.py pkg_resources: v{pkg.version}')

    entries.append(f'- aiohttp v{aiohttp.__version__}')
    uname = platform.uname()
    entries.append('- system info: {0.system} {0.release} {0.version}'.format(uname))
    print('\n'.join(entries))

def core(parser: argparse.ArgumentParser, args: argparse.Namespace) -> None:
    if args.version:
        show_version()
    else:
        parser.print_help()



def parse_args() -> Tuple[argparse.ArgumentParser, argparse.Namespace]:
    parser = argparse.ArgumentParser(prog='quaver', description='Tools for helping with quaver.py')
    parser.add_argument('-v', '--version', action='store_true', help='shows the library version')
    parser.set_defaults(func=core)
    return parser, parser.parse_args()


def main() -> None:
    parser, args = parse_args()
    args.func(parser, args)


if __name__ == '__main__':
    main()